@@ -24,17 +24,23 @@ static GtkSocketClass *parent_class = NULL;
 static void
 bonobo_socket_finalize (GObject *object)
 {
-	dprintf ("bonobo_socket_finalize\n");
+	dprintf ("bonobo_socket_finalize %p\n", object);
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+gboolean
+bonobo_socket_disposed (BonoboSocket *socket)
+{
+	return (socket->frame == NULL);
+}
+
 static void
 bonobo_socket_dispose (GObject *object)
 {
 	BonoboSocket *socket = (BonoboSocket *) object;
 
-	dprintf ("bonobo_socket_dispose \n");
+	dprintf ("bonobo_socket_dispose %p\n", object);
 
 	if (socket->frame) {
 		BonoboObject *object = BONOBO_OBJECT (socket->frame);
@@ -57,13 +63,16 @@ bonobo_socket_realize (GtkWidget *widget)
 
 	socket = BONOBO_SOCKET (widget);
 
-	dprintf ("bonobo_socket_realize\n");
+	dprintf ("bonobo_socket_realize %p\n", widget);
 
 	if (GTK_WIDGET_CLASS (parent_class)->realize)
 		(* GTK_WIDGET_CLASS (parent_class)->realize) (widget);
 
-	if (socket->frame)
-		bonobo_control_frame_set_remote_window (socket->frame);
+	if (socket->frame) {
+		g_object_ref (G_OBJECT (socket->frame));
+		bonobo_control_frame_get_remote_window (socket->frame, NULL);
+		g_object_unref (G_OBJECT (socket->frame));
+	}
 
 	g_assert (GTK_WIDGET_REALIZED (widget));
 }
@@ -80,7 +89,26 @@ bonobo_socket_unrealize (GtkWidget *widget)
 	GTK_WIDGET_UNSET_FLAGS (widget, GTK_REALIZED);
 
 	GTK_WIDGET_CLASS (parent_class)->unrealize (widget);
-	dprintf ("unrealize done %p\n", widget);
+}
+
+static gboolean
+bonobo_socket_expose_event (GtkWidget      *widget,
+			    GdkEventExpose *event)
+{
+	dprintf ("bonobo_socket_expose_event %p (%d, %d), (%d, %d)\n",
+		 widget,
+		 event->area.x, event->area.y,
+		 event->area.width, event->area.height);
+
+#ifdef DEBUG_CONTROL
+	gdk_draw_line (widget->window,
+		       widget->style->black_gc,
+		       event->area.x, event->area.y,
+		       event->area.x + event->area.width,
+		       event->area.y + event->area.height);
+#endif
+
+	return FALSE;
 }
 
 static void
@@ -105,13 +133,11 @@ bonobo_socket_focus_in (GtkWidget     *widget,
 {
 	BonoboSocket *socket = BONOBO_SOCKET (widget);
 
-	if (!socket->frame)
-		return FALSE;
-
-	if (!bonobo_control_frame_get_autoactivate (socket->frame))
-		return FALSE;
-
-	bonobo_control_frame_control_activate (socket->frame);
+	if (socket->frame &&
+	    bonobo_control_frame_get_autoactivate (socket->frame))
+		bonobo_control_frame_control_activate (socket->frame);
+	else
+		dprintf ("No activate on focus in");
 
 	return GTK_WIDGET_CLASS (parent_class)->focus_in_event (widget, focus);
 }
@@ -122,17 +148,27 @@ bonobo_socket_focus_out (GtkWidget     *widget,
 {
 	BonoboSocket *socket = BONOBO_SOCKET (widget);
 
-	if (!socket->frame)
-		return FALSE;
-
-	if (!bonobo_control_frame_get_autoactivate (socket->frame))
-		return FALSE;
-
-	bonobo_control_frame_control_deactivate (socket->frame);
+	if (socket->frame &&
+	    bonobo_control_frame_get_autoactivate (socket->frame))
+		bonobo_control_frame_control_deactivate (socket->frame);
+	else
+		dprintf ("No de-activate on focus out");
 
 	return GTK_WIDGET_CLASS (parent_class)->focus_out_event (widget, focus);
 }
 
+static void
+bonobo_socket_size_allocate (GtkWidget     *widget,
+			     GtkAllocation *allocation)
+{
+	dprintf ("bonobo_socket_size_allocate: (%d, %d), (%d, %d)\n",
+		 allocation->x, allocation->y,
+		 allocation->width, allocation->height);
+	
+	return GTK_WIDGET_CLASS (parent_class)->size_allocate (
+		widget, allocation);
+}
+
 static void
 bonobo_socket_size_request (GtkWidget      *widget,
 			    GtkRequisition *requisition)
@@ -169,14 +205,34 @@ bonobo_socket_size_request (GtkWidget      *widget,
 
 		CORBA_exception_free (ev);
 	}
+
+	dprintf ("bonobo_socket_size_request: %d, %d\n",
+		 requisition->width, requisition->height);
+}
+
+static void
+bonobo_socket_show_all (GtkWidget *widget)
+{
+	/* Do nothing - we don't want this to
+	 * propagate to an in-proc plug */
+}
+
+static gboolean
+bonobo_socket_plug_removed (GtkSocket *socket)
+{
+	dprintf ("bonobo_socket_plug_removed %p\n", socket);
+
+	return TRUE;
 }
 
 static void
 bonobo_socket_class_init (GObjectClass *klass)
 {
 	GtkWidgetClass *widget_class;
+	GtkSocketClass *socket_class;
 
 	widget_class = (GtkWidgetClass*) klass;
+	socket_class = (GtkSocketClass*) klass;
 
 	parent_class = g_type_class_peek_parent (klass);
 
@@ -189,12 +245,17 @@ bonobo_socket_class_init (GObjectClass *klass)
 	widget_class->focus_in_event  = bonobo_socket_focus_in;
 	widget_class->focus_out_event = bonobo_socket_focus_out;
 	widget_class->size_request    = bonobo_socket_size_request;
+	widget_class->size_allocate   = bonobo_socket_size_allocate;
+	widget_class->expose_event    = bonobo_socket_expose_event;
+	widget_class->show_all        = bonobo_socket_show_all;
+
+	socket_class->plug_removed    = bonobo_socket_plug_removed;
 }
 
-guint
+GType
 bonobo_socket_get_type ()
 {
-	static guint socket_type = 0;
+	static GType socket_type = 0;
 
 	if (!socket_type) {
 		static const GtkTypeInfo socket_info = {
